# coding: utf-8

from pathlib import Path

from .util import BaseTestInterface, mock_filedialog, mock_warning_window


class Test08SetChannels(BaseTestInterface):

    def testSetChannels(self) -> None:
        """This test checks that the checkboxes in the interface can
        successfully show or hide the channels of the displayed image."""

        # The mock selection window returns the path to one image to load
        # The image was modified so that all three channels have a non-zero
        # value in pixel (0, 0)
        mock_filedialog.file_name = [str(Path(__file__).parent / 'data' /
                                         'image_3.jpg')]
        mock_warning_window.WarningWindow.value = 1
        self._window._select_images()

        # Reading the values of the checkboxes driving the display of the
        # channels
        init_r = self._window.settings.red_channel_bool.get()
        init_g = self._window.settings.green_channel_bool.get()
        init_b = self._window.settings.blue_channel_bool.get()

        # Reading the RGB value of the pixel in position (0, 0)
        r, g, b = (self._window._image_canvas._canvas.image_tk.
                   _PhotoImage__photo.get(0, 0))

        # Checking that the RGB values are consistent with the state of the
        # checkboxes
        self.assertIs(bool(r), init_r)
        self.assertIs(bool(g), init_g)
        self.assertIs(bool(b), init_b)

        # Generating a click on all the checkboxes to invert them
        self._window._red_channel_check_button.invoke()
        self._window._green_channel_check_button.invoke()
        self._window._blue_channel_check_button.invoke()

        # Reading the new RGB value of the pixel in position (0, 0)
        r, g, b = (self._window._image_canvas._canvas.image_tk.
                   _PhotoImage__photo.get(0, 0))

        # Reading the new values of the checkboxes driving the display of the
        # channels
        new_r = self._window.settings.red_channel_bool.get()
        new_g = self._window.settings.green_channel_bool.get()
        new_b = self._window.settings.blue_channel_bool.get()

        # Checking that the state of the checkboxes was successfully inverted
        # by clicking
        self.assertIs(new_r, not init_r)
        self.assertIs(new_g, not init_g)
        self.assertIs(new_b, not init_b)

        # Checking that the RGB values are consistent with the new state of the
        # checkboxes
        self.assertIs(bool(r), new_r)
        self.assertIs(bool(g), new_g)
        self.assertIs(bool(b), new_b)



# coding: utf-8

from pathlib import Path
from threading import Thread
from time import sleep

from .util import (BaseTestInterfaceProcessing, mock_filedialog,
                   mock_warning_window)


class Test12StopProcessImages(BaseTestInterfaceProcessing):

    def _stop_processing(self) -> None:
        """This method sets the _stop_event flag while the processing loop is
        running.

        It emulates a click on the "Stop Processing" button in the interface,
        which cannot be achieved in the particular conditions of unit testing.
        """

        sleep(1)
        self._window._stop_event.set()

    def testStopProcessImages(self) -> None:
        """This test checks that the image processing can be successfully
        interrupted by clicking on the "Stop Processing" button after it was
        started."""

        # The mock selection window returns the paths to three images to load
        mock_filedialog.file_name = [
            str(Path(__file__).parent / 'data' / f'image_{i}.jpg')
            for i in (1, 2, 3)]
        mock_warning_window.WarningWindow.value = 1
        self._window._select_images()

        # Stopping the regular processing Thread
        self._window._stop_thread = True
        sleep(2)
        # Instantiating a Thread that will stop the processing loop later on
        stop_thread = Thread(target=self._stop_thread)
        # Instantiating a Thread that will stop the image processing later on
        click_thread = Thread(target=self._stop_processing)

        # Checking that the image canvas is empty of nuclei
        self.assertEqual(len(self._window._image_canvas._nuclei), 0)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_in_count,
                         0)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_out_count,
                         0)
        # Checking that the files table is empty of nuclei and fibers
        for i in range(3):
            self.assertEqual(
                len(self._window._files_table.table_items.entries[i].nuclei),
                0)
            self.assertEqual((self._window._files_table.table_items.entries[i].
                              nuclei.nuclei_in_count), 0)
            self.assertEqual((self._window._files_table.table_items.entries[i].
                              nuclei.nuclei_out_count), 0)
            self.assertEqual(
                len(self._window._files_table.table_items.entries[i].fibers),
                0)
            self.assertEqual((self._window._files_table.table_items.entries[i].
                              fibers.area), 0)

        # Invoking the button for starting the image processing
        self._window._process_images_button.invoke()

        # Starting the Threads for later stopping the processing loop
        self._window._stop_thread = False
        stop_thread.start()
        click_thread.start()
        # Starting the processing loop in the main Thread rather than in a
        # separate one
        self._window._process_thread()

        # Checking that the image canvas is still empty of nuclei due to the
        # processing being interrupted
        self.assertEqual(len(self._window._image_canvas._nuclei), 0)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_in_count,
                         0)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_out_count,
                         0)

        # Checking that the files table is still empty of nuclei and fibers due
        # to the processing being interrupted
        for i in range(3):
            self.assertEqual(
                len(self._window._files_table.table_items.entries[i].nuclei),
                0)
            self.assertEqual((self._window._files_table.table_items.entries[i].
                              nuclei.nuclei_in_count), 0)
            self.assertEqual((self._window._files_table.table_items.entries[i].
                              nuclei.nuclei_out_count), 0)
            self.assertEqual(
                len(self._window._files_table.table_items.entries[i].fibers),
                0)
            self.assertEqual((self._window._files_table.table_items.entries[i].
                              fibers.area), 0)



# coding: utf-8

from pathlib import Path
from PIL import Image

from .util import BaseTestInterface, mock_filedialog, mock_warning_window


class Test09DeleteImage(BaseTestInterface):

    def testDeleteImage(self) -> None:
        """This test checks that an image can be successfully deleted from the
        files table and the image canvas by clicking on the delete button of
        the files table."""

        # The mock selection window returns the paths to two images to load
        mock_filedialog.file_name = [
            str(Path(__file__).parent / 'data' / f'image_{i}.jpg')
            for i in (1, 2)]
        mock_warning_window.WarningWindow.value = 1
        self._window._select_images()

        # Checking that the first image is displayed in the image canvas
        self.assertIsInstance(self._window._image_canvas._image, Image.Image)
        self.assertIsInstance(self._window._image_canvas._image_path, Path)
        self.assertEqual(self._window._image_canvas._image_path.name,
                         'image_1.jpg')

        # Checking that two images are referenced in the files table
        self.assertEqual(len(self._window._files_table.table_items), 2)
        self.assertEqual(
            self._window._files_table.table_items.current_entry.path.name,
            'image_1.jpg')
        self.assertCountEqual(
            [item.path.name for item in self._window._files_table.table_items],
            [f'image_{i}.jpg' for i in (1, 2)])

        # Generating a delete event on the first image in the files table
        (self._window._files_table.table_items.current_entry.graph_elt.
         close_button.invoke())

        # Checking that the second image is now displayed in the image canvas
        self.assertIsInstance(self._window._image_canvas._image, Image.Image)
        self.assertIsInstance(self._window._image_canvas._image_path, Path)
        self.assertEqual(self._window._image_canvas._image_path.name,
                         'image_2.jpg')

        # Checking that only one image is left in the files table
        self.assertEqual(len(self._window._files_table.table_items), 1)
        self.assertEqual(
            self._window._files_table.table_items.current_entry.path.name,
            'image_2.jpg')

        # Generating a delete event on the second image in the files table
        (self._window._files_table.table_items.current_entry.graph_elt.
         close_button.invoke())

        # Checking that no image is displayed in the image canvas
        self.assertIsNone(self._window._image_canvas._image)
        self.assertIsNone(self._window._image_canvas._image_path)

        # Checking that no image is left in the files table
        self.assertEqual(len(self._window._files_table.table_items), 0)



# coding: utf-8

from tkinter import Toplevel, ttk, Scale
from screeninfo import get_monitors


class SettingsWindow(Toplevel):
    """Popup window for setting the parameters of the software."""

    def __init__(self, main_window) -> None:
        """Creates the window, arranges its layout and displays it.

        Args:
            main_window: The main window of the project.
        """

        super().__init__(main_window)

        # Setting the variables
        self._main_window = main_window
        self._settings = self._main_window.settings

        # Setting window properties
        self.resizable(False, False)
        self.grab_set()
        self.title("Settings")

        # Finish setting up the window
        self._set_layout()
        self.update()
        self._center()

    def destroy(self) -> None:
        """Before exiting, saves the settings to a file in the application
        folder."""

        if self._main_window.app_folder is not None:
            self._main_window.save_settings(self._main_window.app_folder)
        self._main_window._settings_window = None
        super().destroy()

    def _set_layout(self) -> None:
        """Creates the buttons and sliders, places them and displays them."""

        # General layout
        self._frame = ttk.Frame(self, padding="20 20 20 20")
        self._frame.grid(sticky='NESW')

        # Buttons for selecting the nuclei channel
        self._nuclei_colour_label = ttk.Label(self._frame,
                                              text="Nuclei Channel :")
        self._nuclei_colour_label.grid(column=0, row=0, sticky='NE',
                                       padx=(0, 10))
        self._nuclei_colour_r1 = ttk.Radiobutton(
            self._frame, text="Blue", variable=self._settings.nuclei_colour,
            value="blue")
        self._nuclei_colour_r1.grid(column=1, row=0, sticky='NW')
        self._nuclei_colour_r2 = ttk.Radiobutton(
            self._frame, text="Green", variable=self._settings.nuclei_colour,
            value="green")
        self._nuclei_colour_r2.grid(column=1, row=1, sticky='NW')
        self._nuclei_colour_r3 = ttk.Radiobutton(
            self._frame, text="Red", variable=self._settings.nuclei_colour,
            value="red")
        self._nuclei_colour_r3.grid(column=1, row=2, sticky='NW')

        # Buttons for selecting the fibers channel
        self._fiber_colour_label = ttk.Label(self._frame,
                                             text="Fiber Channel :")
        self._fiber_colour_label.grid(column=0, row=3, sticky='NE',
                                      pady=(10, 0), padx=(0, 10))
        self._fiber_colour_r1 = ttk.Radiobutton(
            self._frame, text="Blue", variable=self._settings.fiber_colour,
            value="blue")
        self._fiber_colour_r1.grid(column=1, row=3, sticky='NW', pady=(10, 0))
        self._fiber_colour_r2 = ttk.Radiobutton(
            self._frame, text="Green", variable=self._settings.fiber_colour,
            value="green")
        self._fiber_colour_r2.grid(column=1, row=4, sticky='NW')
        self._fiber_colour_r3 = ttk.Radiobutton(
            self._frame, text="Red", variable=self._settings.fiber_colour,
            value="red")
        self._fiber_colour_r3.grid(column=1, row=5, sticky='NW')

        # Buttons to chose whether to save the overlay images or not
        self._overlay_label = ttk.Label(self._frame,
                                        text='Save Images with Overlay :')
        self._overlay_label.grid(column=0, row=11, sticky='NE', pady=(10, 0),
                                 padx=(0, 10))
        self._overlay_on_button = ttk.Radiobutton(
            self._frame, text="On", variable=self._settings.save_overlay,
            value=1)
        self._overlay_on_button.grid(column=1, row=11, sticky='NW',
                                     pady=(10, 0))
        self._overlay_off_button = ttk.Radiobutton(
            self._frame, text="Off", variable=self._settings.save_overlay,
            value=0)
        self._overlay_off_button.grid(column=1, row=12, sticky='NW')

        # Slider to adjust the minimum intensity for fiber detection
        self._min_fiber_int_label = ttk.Label(self._frame,
                                              text='Minimum fiber intensity :')
        self._min_fiber_int_label.grid(column=0, row=13, sticky='E',
                                       pady=(10, 0), padx=(0, 10))

        self._minimum_fiber_intensity_slider_frame = ttk.Frame(self._frame)

        self._min_fib_int_slide_val_label = ttk.Label(
            self._minimum_fiber_intensity_slider_frame,
            textvariable=self._settings.minimum_fiber_intensity,
            width=3)
        self._min_fib_int_slide_val_label.pack(
            side='left', anchor='w', fill='none', expand=False, padx=(0, 20))

        self._min_fib_int_slider = Scale(
            self._minimum_fiber_intensity_slider_frame, from_=0, to=100,
            orient="horizontal",
            variable=self._settings.minimum_fiber_intensity, showvalue=False,
            length=150, tickinterval=20)

        self._min_fib_int_slider.pack(side='left', anchor='w', fill='none',
                                      expand=False)

        self._minimum_fiber_intensity_slider_frame.grid(
            column=1, row=13, sticky='NW', pady=(10, 0))

        # Slider to adjust the maximum intensity for fiber detection
        self._max_fiber_int_label = ttk.Label(self._frame,
                                              text='Maximum fiber intensity :')
        self._max_fiber_int_label.grid(column=0, row=14, sticky='E',
                                       pady=(10, 0), padx=(0, 10))

        self._maximum_fiber_intensity_slider_frame = ttk.Frame(self._frame)

        self._max_fib_int_slide_val_label = ttk.Label(
            self._maximum_fiber_intensity_slider_frame,
            textvariable=self._settings.maximum_fiber_intensity, width=3)
        self._max_fib_int_slide_val_label.pack(
            side='left', anchor='w', fill='none', expand=False, padx=(0, 20))

        self._max_fib_int_slider = Scale(
            self._maximum_fiber_intensity_slider_frame, from_=155, to=255,
            orient="horizontal",
            variable=self._settings.maximum_fiber_intensity, showvalue=False,
            length=150, tickinterval=25)
        self._max_fib_int_slider.pack(side='left', anchor='w', fill='none',
                                      expand=False)

        self._maximum_fiber_intensity_slider_frame.grid(
            column=1, row=14, sticky='NW', pady=(10, 0))

        # Slider to adjust the minimum intensity for nuclei detection
        self._min_nucleus_int_label = ttk.Label(
            self._frame, text='Minimum nucleus intensity :')
        self._min_nucleus_int_label.grid(column=0, row=15, sticky='E',
                                         pady=(10, 0), padx=(0, 10))

        self._minimum_nuclei_intensity_slider_frame = ttk.Frame(self._frame)

        self._min_nuc_int_slide_val_label = ttk.Label(
            self._minimum_nuclei_intensity_slider_frame,
            textvariable=self._settings.minimum_nucleus_intensity, width=3)
        self._min_nuc_int_slide_val_label.pack(
            side='left', anchor='w', fill='none', expand=False, padx=(0, 20))

        self._min_nuc_int_slider = Scale(
            self._minimum_nuclei_intensity_slider_frame, from_=0, to=100,
            orient="horizontal",
            variable=self._settings.minimum_nucleus_intensity,
            showvalue=False, length=150, tickinterval=20)
        self._min_nuc_int_slider.pack(side='left', anchor='w', fill='none',
                                      expand=False)

        self._minimum_nuclei_intensity_slider_frame.grid(
            column=1, row=15, sticky='NW', pady=(10, 0))

        # Slider to adjust the maximum intensity for nuclei detection
        self._max_nucleus_int_label = ttk.Label(
            self._frame, text='Maximum nucleus intensity :')
        self._max_nucleus_int_label.grid(column=0, row=16, sticky='E',
                                         pady=(10, 0), padx=(0, 10))

        self._maximum_nuclei_intensity_slider_frame = ttk.Frame(self._frame)

        self._max_nuc_int_slide_val_label = ttk.Label(
            self._maximum_nuclei_intensity_slider_frame,
            textvariable=self._settings.maximum_nucleus_intensity, width=3)
        self._max_nuc_int_slide_val_label.pack(
            side='left', anchor='w', fill='none', expand=False, padx=(0, 20))

        self._max_nuc_int_slider = Scale(
            self._maximum_nuclei_intensity_slider_frame, from_=155, to=255,
            orient="horizontal",
            variable=self._settings.maximum_nucleus_intensity,
            showvalue=False, length=150, tickinterval=25)
        self._max_nuc_int_slider.pack(side='left', anchor='w', fill='none',
                                      expand=False)

        self._maximum_nuclei_intensity_slider_frame.grid(
            column=1, row=16, sticky='NW', pady=(10, 0))

        # Slider to adjust the minimum nucleus diameter
        self._min_nucleus_diam_label = ttk.Label(
            self._frame, text='Minimum nucleus diameter (px) :')
        self._min_nucleus_diam_label.grid(
            column=0, row=17, sticky='E', pady=(10, 0), padx=(0, 10))

        self._diameter_slider_frame = ttk.Frame(self._frame)

        self._min_nuc_diam_slide_val_label = ttk.Label(
            self._diameter_slider_frame,
            textvariable=self._settings.minimum_nuc_diameter, width=3)
        self._min_nuc_diam_slide_val_label.pack(
            side='left', anchor='w', fill='none', expand=False, padx=(0, 20))

        self._min_nuc_diam_slider = Scale(
            self._diameter_slider_frame, from_=0, to=100,
            variable=self._settings.minimum_nuc_diameter,
            orient="horizontal", length=150, showvalue=False,
            tickinterval=25)
        self._min_nuc_diam_slider.pack(side='left', anchor='w',
                                       fill='none', expand=False)

        self._diameter_slider_frame.grid(column=1, row=17, sticky='NW',
                                         pady=(10, 0))

        # Slider to adjust the minimum nucleus count
        self._min_nuclei_count_label = ttk.Label(
            self._frame, text='Minimum nuclei count :')
        self._min_nuclei_count_label.grid(
            column=0, row=18, sticky='E', pady=(10, 0), padx=(0, 10))

        self._count_slider_frame = ttk.Frame(self._frame)

        self._count_slide_val_label = ttk.Label(
            self._count_slider_frame,
            textvariable=self._settings.minimum_nuclei_count, width=3)
        self._count_slide_val_label.pack(
            side='left', anchor='w', fill='none', expand=False, padx=(0, 20))

        self._count_slider = Scale(
            self._count_slider_frame, from_=0, to=8,
            variable=self._settings.minimum_nuclei_count,
            orient="horizontal", length=150, showvalue=False,
            tickinterval=1)
        self._count_slider.pack(side='left', anchor='w',
                                fill='none', expand=False)

        self._count_slider_frame.grid(column=1, row=18, sticky='NW',
                                      pady=(10, 0))

    def _center(self) -> None:
        """Centers the popup window on the currently used monitor."""

        # Getting the current monitor
        monitors = get_monitors()
        candidates = [monitor for monitor in monitors if
                      0 <= self.winfo_x() - monitor.x <= monitor.width and
                      0 <= self.winfo_y() - monitor.y <= monitor.height]
        current_monitor = candidates[0] if candidates else monitors[0]

        # Getting the parameters of interest
        scr_width = current_monitor.width
        scr_height = current_monitor.height
        x_offset = current_monitor.x
        y_offset = current_monitor.y
        height = self.winfo_height()
        width = self.winfo_width()

        # Actually centering the window
        self.geometry('+%d+%d' % (x_offset + (scr_width - width) / 2,
                                  y_offset + (scr_height - height) / 2))



# coding: utf-8

from pathlib import Path

from .util import BaseTestInterface, mock_filedialog, mock_warning_window


class Test04ClickTable(BaseTestInterface):

    def testClickTable(self) -> None:
        """This test checks that the currently selected and displayed image is
        updated when the user clicks on another entry of the files table."""

        # The mock selection window returns the paths to three images to load
        mock_filedialog.file_name = [
            str(Path(__file__).parent / 'data' / f'image_{i}.jpg')
            for i in (1, 2, 3)]
        mock_warning_window.WarningWindow.value = 1
        self._window._select_images()

        # Checking that the first image is displayed in the image canvas
        self.assertEqual(self._window._image_canvas._image_path.name,
                         'image_1.jpg')
        self.assertEqual(
            self._window._files_table.table_items.current_entry.path.name,
            'image_1.jpg')

        # Generating a click on the third entry of the files table
        (self._window._files_table.table_items.entries[2].graph_elt.
         event_generate('<ButtonPress-1>', when="now"))

        # Checking that the third image is displayed in the image canvas
        self.assertEqual(self._window._image_canvas._image_path.name,
                         'image_3.jpg')
        self.assertEqual(
            self._window._files_table.table_items.current_entry.path.name,
            'image_3.jpg')



# coding: utf-8

from pathlib import Path
from platform import system, python_version_tuple
from unittest import skipIf
import os

from .util import BaseTestInterface, mock_filedialog, mock_warning_window

condition = (os.getenv('MYOFINDER_GITHUB_ACTION', 0) == '1'
             and system() == 'Windows'
             and int(python_version_tuple()[1]) < 9)


@skipIf(condition,
        "For some reason, this test fails on Windows with a Python version "
        "anterior to 3.8. It was manually checked that the drag feature was "
        "working as expected in the interface nevertheless.")
class Test07Drag(BaseTestInterface):

    def testDrag(self) -> None:
        """This test checks that a mousewheel click-and-drag successfully moves
        the image in the image canvas."""

        # The mock selection window returns the path to one image to load
        mock_filedialog.file_name = [str(Path(__file__).parent / 'data' /
                                         'image_1.jpg')]
        mock_warning_window.WarningWindow.value = 1
        self._window._select_images()

        # Reading the initial coordinates of the upper-left corner of the image
        # on the image canvas
        init_x = self._window._image_canvas._canvas.canvasx(0)
        init_y = self._window._image_canvas._canvas.canvasy(0)

        # The zoom commands differ on Linux, Windows, and macOS
        # Generating five zoom-in events with the mousewheel
        for _ in range(5):
            if system() == "Linux":
                self._window._image_canvas._canvas.event_generate(
                    '<4>', when="now", x=100, y=100)
            else:
                self._window._image_canvas._canvas.event_generate(
                    '<MouseWheel>', when="now", x=100, y=100, delta=1)

        # checking that the coordinates of the upper-left corner of the image
        # in the image canvas have changed
        self.assertGreater(self._window._image_canvas._canvas.canvasx(0),
                           init_x)
        self.assertGreater(self._window._image_canvas._canvas.canvasy(0),
                           init_y)
        # Reading the new coordinates of the upper-left corner of the image on
        # the image canvas
        new_x = self._window._image_canvas._canvas.canvasx(0)
        new_y = self._window._image_canvas._canvas.canvasy(0)

        # Generating a mousewheel click-and-drag event on the image canvas
        self._window._image_canvas._canvas.event_generate(
            '<ButtonPress-2>', when="now", x=100, y=100)
        self._window._image_canvas._canvas.event_generate(
            '<B2-Motion>', when="now", x=50, y=50)

        # Checking that the position of the image in the image canvas has been
        # updated with the correct value
        self.assertEqual(self._window._image_canvas._canvas.canvasx(0) - new_x,
                         50)
        self.assertEqual(self._window._image_canvas._canvas.canvasy(0) - new_y,
                         50)



# coding: utf-8

from tkinter import Tk, Canvas, TclError
from PIL import ImageTk, Image
from screeninfo import get_monitors
from time import sleep
import importlib.resources as resources


class SplashWindow(Tk):
    """Window displayed while loading the software.

    It gives information about the loading status.
    """

    def __init__(self) -> None:
        """Sets a few instance attributes."""

        super().__init__()
        # On macOS, some Tkinter versions crash on this call
        try:
            self.overrideredirect(True)
        except TclError:
            pass
        self.grab_set()

        ref = (resources.files('myofinder') / 'app_images'
               / 'splash_background.png')
        with resources.as_file(ref) as path:
            self._image = Image.open(str(path))

    def resize_image(self) -> None:
        """Centers the window on the monitor currently in use."""

        size_factor = 0.35

        # Getting the current monitor
        monitors = get_monitors()
        candidates = [monitor for monitor in monitors if
                      0 <= self.winfo_x() - monitor.x <= monitor.width and
                      0 <= self.winfo_y() - monitor.y <= monitor.height]
        current_monitor = candidates[0] if candidates else monitors[0]

        # Getting the parameters of interest
        scr_width = current_monitor.width
        scr_height = current_monitor.height
        x_offset = current_monitor.x
        y_offset = current_monitor.y
        img_ratio = self._image.width / self._image.height
        scr_ratio = scr_width / scr_height

        # Resizing the window, taking into account the screen proportions
        if img_ratio > scr_ratio:
            self._image = self._image.resize(
                (int(scr_width * size_factor),
                 int(scr_width * size_factor / img_ratio)),
                Image.LANCZOS)
            self.geometry('%dx%d+%d+%d' % (
                int(scr_width * size_factor),
                int(scr_width * size_factor / img_ratio),
                x_offset + scr_width * (1 - size_factor) / 2,
                y_offset + (scr_height - int(scr_width * size_factor /
                                             img_ratio)) / 2))

        else:
            self._image = self._image.resize(
                (int(scr_height * size_factor * img_ratio),
                 int(scr_height * size_factor)),
                Image.LANCZOS)
            self.geometry('%dx%d+%d+%d' % (
                int(scr_height * size_factor * img_ratio),
                int(scr_height * size_factor),
                x_offset + (scr_width - int(scr_height * size_factor *
                                            img_ratio)) / 2,
                y_offset + scr_height * (1 - size_factor) / 2))

    def display(self):
        """Sets the layout of the window, displays it, and loads the modules.

        Returns:
            An instance of the image segmentation class.
        """

        # Sets the background of the window
        image_tk = ImageTk.PhotoImage(self._image)
        canvas = Canvas(self, bg="brown")
        canvas.create_image(0, 0, image=image_tk, anchor="nw")

        # Sets the static text of the window
        canvas.create_text(
            20, int(0.70 * self._image.height),
            anchor='w',
            text="MyoFInDer - A project of the Tissue Engineering Lab at "
                 "KU Leuven\nBy Antoine Weisrock, Rebecca Wüst and "
                 "Maria Olenic",
            fill="white",
            font='Helvetica 7 bold',
            width=self._image.width - 40)

        # Sets the dynamic text of the window
        label = canvas.create_text(
            20, int(0.9 * self._image.height),
            anchor="w",
            text='Importing dependencies...',
            fill="white",
            font='Helvetica 7 bold',
            width=self._image.width - 40)

        # Finish setting the layout
        canvas.pack(fill="both", expand=True)
        self.update()

        from ..image_segmentation import ImageSegmentation

        # Load the image segmentation and update the display
        canvas.itemconfig(label, text="Initializing Mesmer...")
        self.update()
        segmentation = ImageSegmentation()

        # Update the display before starting the software
        canvas.itemconfig(label, text="Starting program...")
        self.update()

        sleep(1)

        return segmentation



# coding: utf-8

from pathlib import Path

from .util import BaseTestInterface, mock_filedialog, mock_warning_window


class Test05ClickImage(BaseTestInterface):

    def testClickImage(self) -> None:
        """This test checks that the nuclei are correctly added, inverted, or
        deleted on the image canvas when left- or right-clicking on it, and
        checks that the files table is updated accordingly."""

        # The mock selection window returns the path to one image to load
        mock_filedialog.file_name = [str(Path(__file__).parent / 'data' /
                                         'image_1.jpg')]
        mock_warning_window.WarningWindow.value = 1
        self._window._select_images()

        # Checking that the image canvas and file table are empty of nuclei
        self.assertEqual(len(self._window._image_canvas._nuclei), 0)
        self.assertEqual(
            len(self._window._files_table.table_items.current_entry.nuclei), 0)

        # Checking that the image canvas and file table are empty of fibers
        self.assertEqual(len(self._window._image_canvas._fibers), 0)
        self.assertEqual(
            len(self._window._files_table.table_items.current_entry.fibers), 0)

        # Generating 10 left-clicks on the image canvas
        for i in range(1, 11):
            self._window._image_canvas._canvas.event_generate(
                '<ButtonPress-1>', when="now", x=10 * i, y=10 * i)
            self._window._image_canvas._canvas.event_generate(
                '<ButtonRelease-1>',  when="now", x=10 * i, y=10 * i)

        # Checking that the image canvas and the files table have the correct
        # nuclei count
        self.assertEqual(len(self._window._image_canvas._nuclei), 10)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_in_count,
                         0)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_out_count,
                         10)
        self.assertEqual(
            len(self._window._files_table.table_items.current_entry.nuclei),
            10)
        self.assertEqual((self._window._files_table.table_items.current_entry.
                          nuclei.nuclei_in_count), 0)
        self.assertEqual((self._window._files_table.table_items.current_entry.
                          nuclei.nuclei_out_count), 10)

        # Generating left-clicks on 5 of the 10 created nuclei
        for i in range(1, 6):
            self._window._image_canvas._canvas.event_generate(
                '<ButtonPress-1>', when="now", x=20 * i, y=20 * i)
            self._window._image_canvas._canvas.event_generate(
                '<ButtonRelease-1>',  when="now", x=20 * i, y=20 * i)

        # Checking that the corresponding nuclei have been inverted on the
        # image canvas and the files table
        self.assertEqual(len(self._window._image_canvas._nuclei), 10)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_in_count,
                         5)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_out_count,
                         5)
        self.assertEqual(
            len(self._window._files_table.table_items.current_entry.nuclei),
            10)
        self.assertEqual((self._window._files_table.table_items.current_entry.
                          nuclei.nuclei_in_count), 5)
        self.assertEqual((self._window._files_table.table_items.current_entry.
                          nuclei.nuclei_out_count), 5)

        # Generating right-clicks on 5 of the 10 created nuclei
        for i in range(1, 6):
            self._window._image_canvas._canvas.event_generate(
                '<ButtonPress-3>', when="now", x=20 * i, y=20 * i)
            self._window._image_canvas._canvas.event_generate(
                '<ButtonRelease-3>',  when="now", x=20 * i, y=20 * i)

        # Checking that the corresponding nuclei have been deleted on the
        # image canvas and the files table
        self.assertEqual(len(self._window._image_canvas._nuclei), 5)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_in_count,
                         0)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_out_count,
                         5)
        self.assertEqual(
            len(self._window._files_table.table_items.current_entry.nuclei),
            5)
        self.assertEqual((self._window._files_table.table_items.current_entry.
                          nuclei.nuclei_in_count), 0)
        self.assertEqual((self._window._files_table.table_items.current_entry.
                          nuclei.nuclei_out_count), 5)

        # Generating a left click-and-drag over the 5 remaining nuclei
        self._window._image_canvas._canvas.event_generate(
            '<ButtonPress-1>', when="now", x=5, y=5)
        self._window._image_canvas._canvas.event_generate(
            '<B1-Motion>', when="now", x=105, y=105)
        self._window._image_canvas._canvas.event_generate(
            '<ButtonRelease-1>', when="now", x=105, y=105)

        # Checking that the corresponding nuclei have been inverted on the
        # image canvas and the files table
        self.assertEqual(len(self._window._image_canvas._nuclei), 5)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_in_count,
                         5)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_out_count,
                         0)
        self.assertEqual(
            len(self._window._files_table.table_items.current_entry.nuclei),
            5)
        self.assertEqual((self._window._files_table.table_items.current_entry.
                          nuclei.nuclei_in_count), 5)
        self.assertEqual((self._window._files_table.table_items.current_entry.
                          nuclei.nuclei_out_count), 0)

        # Generating a right click-and-drag over the 5 remaining nuclei
        self._window._image_canvas._canvas.event_generate(
            '<ButtonPress-3>', when="now", x=5, y=5)
        self._window._image_canvas._canvas.event_generate(
            '<B3-Motion>', when="now", x=105, y=105)
        self._window._image_canvas._canvas.event_generate(
            '<ButtonRelease-3>', when="now", x=105, y=105)

        # Checking that the corresponding nuclei have been deleted on the
        # image canvas and the files table
        self.assertEqual(len(self._window._image_canvas._nuclei), 0)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_in_count,
                         0)
        self.assertEqual(self._window._image_canvas._nuclei.nuclei_out_count,
                         0)
        self.assertEqual(
            len(self._window._files_table.table_items.current_entry.nuclei),
            0)
        self.assertEqual((self._window._files_table.table_items.current_entry.
                          nuclei.nuclei_in_count), 0)
        self.assertEqual((self._window._files_table.table_items.current_entry.
                          nuclei.nuclei_out_count), 0)



# coding: utf-8

"""This file contains the executable code that allows to run the MyoFInDer
interface."""

from . import MainWindow
import logging
from sys import stdout, exit
from pathlib import Path
import argparse
from platform import system
from typing import Optional


def main():
    """This is the main method called for running the MyoFInDer application.

    It first parses the command-line arguments, then initializes the logger,
    and finally starts the application window.
    """

    # Parser for parsing the command line arguments of the script
    parser = argparse.ArgumentParser(
        description="Starts MyoFInDer, with optional flags for specifying "
                    "whether to record log messages and whether the module was"
                    " started by the application or independently.")
    parser.add_argument('-n', '--nolog', action='store_false',
                        help="If provided, the log messages won't be displayed"
                             " and recorded. Otherwise, they are by default.")
    parser.add_argument('-t', '--test', action='store_true',
                        help="If provided, the main window is created but its "
                             "main loop is never started and the window is "
                             "destroyed as soon as it is initialized.")
    parser.add_argument('-f', '--app-folder', action='store', nargs=1,
                        type=Path, required=False, dest='app_folder',
                        help="If provided, should contain the path to the "
                             "folder where to store the log messages and the "
                             "settings file.")
    args = parser.parse_args()

    # Retrieving the command-line arguments
    log: bool = args.nolog
    test: bool = args.test
    app_folder: Optional[Path]
    if args.app_folder is None:
        app_folder = None
    else:
        app_folder = args.app_folder[0]

    # Setting the path to the application folder if not already provided
    if app_folder is None:
        if system() in ('Linux', 'Darwin'):
            app_folder = Path.home() / '.MyoFInDer'
        elif system() == 'Windows':
            app_folder = (Path.home() / 'AppData' / 'Local' / 'MyoFInDer')

    # Creating the application folder, if needed
    if app_folder is not None:
        try:
            app_folder.mkdir(parents=True, exist_ok=True)
        except FileNotFoundError:
            app_folder = None

    # Setting up the logger
    logger = logging.getLogger("MyoFInDer")
    logger.setLevel(logging.INFO)

    # Setting up the handlers only if logging is enabled
    if log:
        handler_console = logging.StreamHandler(stream=stdout)
        handler_console.setLevel(logging.INFO)

        # Setting up the formatter
        formatter = logging.Formatter('%(asctime)s %(name)-8s %(message)s')
        handler_console.setFormatter(formatter)

        logger.addHandler(handler_console)

        # Setting up the file handler only if a location is provided
        if app_folder is not None:
            handler_file = logging.FileHandler(app_folder / 'logs.txt',
                                               mode='w')
            handler_file.setLevel(logging.INFO)
            handler_file.setFormatter(formatter)
            logger.addHandler(handler_file)

    # Normal workflow
    try:
        logger.log(logging.INFO, "Launching MyoFInDer")
        window = MainWindow(app_folder)
        # Normal operation mode, start the mainloop
        if not test:
            window.mainloop()
        # Test mode, destroy the main window right away
        else:
            window.safe_destroy()
        logger.log(logging.INFO, "MyoFInDer terminated gracefully")

    # Displaying the exception and waiting for the user to close the console
    except (Exception,) as exc:
        logger.exception("MyoFInDer encountered an error while running !",
                         exc_info=exc)
        # Only leave the console on if logging is enabled
        if log:
            print("\n\nPress ENTER to exit ...")
            input()


if __name__ == "__main__":

    exit(main())



# coding: utf-8

from pathlib import Path
from platform import system

from .util import BaseTestInterface, mock_filedialog, mock_warning_window


class Test06ZoomInOut(BaseTestInterface):

    def testZoomInOut(self) -> None:
        """This test checks that the image in the image canvas is correctly
        resized when using the mousewheel."""

        # The mock selection window returns the path to one image to load
        mock_filedialog.file_name = [str(Path(__file__).parent / 'data' /
                                         'image_1.jpg')]
        mock_warning_window.WarningWindow.value = 1
        self._window._select_images()

        # Reading the scale and dimension of the image before any interaction
        scale_0 = self._window._image_canvas._img_scale
        width_0 = self._window._image_canvas._canvas.image_tk.width()
        height_0 = self._window._image_canvas._canvas.image_tk.height()

        # Checking that the displayed image is by default not zoomed in or out
        self.assertEqual(self._window._image_canvas._can_scale, 1.0)
        self.assertEqual(self._window._image_canvas._current_zoom, 0)

        # The zoom commands differ on Linux, Windows, and macOS
        # Generating a single zoom-in event with the mousewheel
        if system() == "Linux":
            self._window._image_canvas._canvas.event_generate(
                '<4>', when="now", x=5, y=5)
        else:
            self._window._image_canvas._canvas.event_generate(
                '<MouseWheel>', when="now", x=5, y=5, delta=1)

        # Checking that the zoom was correctly applied in the image canvas
        self.assertAlmostEqual(self._window._image_canvas._img_scale, scale_0 *
                               self._window._image_canvas._delta)
        self.assertEqual(self._window._image_canvas._can_scale,
                         self._window._image_canvas._delta)
        self.assertEqual(self._window._image_canvas._current_zoom, 1)
        self.assertGreater(self._window._image_canvas._canvas.image_tk.width(),
                           width_0)
        self.assertGreater(
            self._window._image_canvas._canvas.image_tk.height(), height_0)

        # Generating five zoom-in events with the mousewheel
        for _ in range(5):
            if system() == "Linux":
                self._window._image_canvas._canvas.event_generate(
                    '<4>', when="now", x=5, y=5)
            else:
                self._window._image_canvas._canvas.event_generate(
                    '<MouseWheel>', when="now", x=5, y=5, delta=1)

        # Checking that the zoom was correctly applied in the image canvas, and
        # that the limit of 5 zoom levels in enforced
        self.assertAlmostEqual(self._window._image_canvas._img_scale, scale_0 *
                               self._window._image_canvas._delta ** 5)
        self.assertAlmostEqual(self._window._image_canvas._can_scale,
                               self._window._image_canvas._delta ** 5)
        self.assertEqual(self._window._image_canvas._current_zoom, 5)
        self.assertGreater(self._window._image_canvas._canvas.image_tk.width(),
                           width_0)
        self.assertGreater(
            self._window._image_canvas._canvas.image_tk.height(), height_0)

        # Generating 11 zoom-out events with the mousewheel
        for _ in range(11):
            if system() == "Linux":
                self._window._image_canvas._canvas.event_generate(
                    '<5>', when="now", x=5, y=5)
            else:
                self._window._image_canvas._canvas.event_generate(
                    '<MouseWheel>', when="now", x=5, y=5, delta=-1)

        # Checking that the zoom was correctly applied in the image canvas, and
        # that the limit of 5 zoom levels in enforced
        self.assertAlmostEqual(self._window._image_canvas._img_scale, scale_0 *
                               self._window._image_canvas._delta ** -5)
        self.assertAlmostEqual(self._window._image_canvas._can_scale,
                               self._window._image_canvas._delta ** -5)
        self.assertEqual(self._window._image_canvas._current_zoom, -5)
        self.assertLess(self._window._image_canvas._canvas.image_tk.width(),
                        width_0)
        self.assertLess(self._window._image_canvas._canvas.image_tk.height(),
                        height_0)
